@@ -3,7 +3,7 @@
 /* gnome-vfs-mime-info.c - GNOME mime-information implementation.
 
    Copyright (C) 1998 Miguel de Icaza
-   Copyright (C) 2000 Eazel
+   Copyright (C) 2000, 2001 Eazel, Inc.
    All rights reserved.
 
    The Gnome Library is free software; you can redistribute it and/or
@@ -140,13 +140,12 @@ static GHashTable *registered_types_user;
 static void	      reload_if_needed (void);
 static GnomeVFSResult write_back_mime_user_file (void);
 static GnomeVFSResult write_back_keys_user_file (void);
-static gboolean       does_string_contains_caps (const char *string);
 static const char *   gnome_vfs_mime_get_registered_mime_type_key (const char *mime_type, 
 								   const char *key);
 
 
 static gboolean 
-does_string_contains_caps (const char *string)
+does_string_contain_caps (const char *string)
 {
 	const char *temp_c;
 
@@ -964,7 +963,7 @@ set_value_real (const char *mime_type, const char *key, const char *value,
 		return gnome_vfs_result_from_errno ();
 	}
 
-	g_return_val_if_fail (!does_string_contains_caps (mime_type), 
+	g_return_val_if_fail (!does_string_contain_caps (mime_type), 
 			      gnome_vfs_result_from_errno ());
 	
 	if (!gnome_vfs_mime_inited) {
@@ -1061,7 +1060,7 @@ get_value_real (const char *mime_type,
 		return NULL;
 	}
 	
-	g_return_val_if_fail (!does_string_contains_caps (mime_type), 
+	g_return_val_if_fail (!does_string_contain_caps (mime_type), 
 			      NULL);
 
 	reload_if_needed ();
@@ -1087,15 +1086,15 @@ get_value_real (const char *mime_type,
 	
 	value = get_value_from_hash_table (user_hash_table, generic_type, key);
 	if (value != NULL) {
+		g_free (generic_type);
 		return value;
 	}
 	
 	value = get_value_from_hash_table (system_hash_table, generic_type, key);
+	g_free (generic_type);
 	if (value != NULL) {
 		return value;
 	}
-	
-	g_free (generic_type);
 
 	return NULL;
 }
@@ -1132,7 +1131,7 @@ gnome_vfs_mime_type_is_known (const char *mime_type)
 		return FALSE;
 	}
 
-	g_return_val_if_fail (!does_string_contains_caps (mime_type), 
+	g_return_val_if_fail (!does_string_contain_caps (mime_type), 
 			      FALSE);
 
 	if (!gnome_vfs_mime_inited)
@@ -1203,7 +1202,7 @@ gnome_vfs_mime_get_key_list (const char *mime_type)
 	if (mime_type == NULL) {
 		return NULL;
 	}
-	g_return_val_if_fail (!does_string_contains_caps (mime_type), 
+	g_return_val_if_fail (!does_string_contain_caps (mime_type), 
 			      NULL);
 
 	if (!gnome_vfs_mime_inited)
@@ -1294,7 +1293,7 @@ gnome_vfs_mime_get_extensions_list (const char *mime_type)
 	if (mime_type == NULL) {
 		return NULL;
 	}
-	g_return_val_if_fail (!does_string_contains_caps (mime_type), 
+	g_return_val_if_fail (!does_string_contain_caps (mime_type), 
 			      NULL);
 
 
@@ -1364,7 +1363,7 @@ gnome_vfs_mime_get_extensions_string (const char *mime_type)
 	if (mime_type == NULL) {
 		return NULL;
 	}
-	g_return_val_if_fail (!does_string_contains_caps (mime_type), 
+	g_return_val_if_fail (!does_string_contain_caps (mime_type), 
 			      NULL);
 
 
@@ -1731,8 +1730,9 @@ write_back_mime_user_file (void)
 			return gnome_vfs_result_from_errno ();
 		}
 
-		fprintf (file, "# this file was autogenerated by gnome-vfs-mime-info.\n"
-			 "# DO NOT EDIT BY HAND. EVER. OR I'll KILL YA.\n");
+		fprintf (file,
+			 "# This file was autogenerated by gnome-vfs-mime-info.\n"
+			 "# Do not edit by hand.\n");
 
 		g_hash_table_foreach (registered_types_user, 
 				      write_back_mime_user_file_callback,
