@@ -2,6 +2,7 @@
 /* gnome-vfs-uri.c - URI handling for the GNOME Virtual File System.
 
    Copyright (C) 1999 Free Software Foundation
+   Copyright (C) 2000, 2001 Eazel, Inc.
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -331,7 +332,11 @@ set_uri_element (GnomeVFSURI *uri,
 	 * meaning.
 	 */
 	if ( ! (strcmp (uri->method_string, "http") == 0 
-	        || strcmp (uri->method_string, "eazel-services") == 0 )) {
+	        || strcmp (uri->method_string, "eazel-services") == 0
+	        || strcmp (uri->method_string, "ghelp") == 0
+	        || strcmp (uri->method_string, "gnome-help") == 0
+	        || strcmp (uri->method_string, "help") == 0
+		)) {
 
 		escaped_text = gnome_vfs_escape_set (uri->text, ";?&=+$,");
 		g_free (uri->text);
@@ -428,11 +433,14 @@ parse_uri_substring (const gchar *substring, GnomeVFSURI *parent)
 GnomeVFSURI *
 gnome_vfs_uri_new (const gchar *text_uri)
 {
-	return gnome_vfs_uri_new_private (text_uri, FALSE);
+	return gnome_vfs_uri_new_private (text_uri, FALSE, FALSE, TRUE);
 }
 
 GnomeVFSURI *
-gnome_vfs_uri_new_private (const gchar *text_uri, gboolean allow_unknown_methods)
+gnome_vfs_uri_new_private (const gchar *text_uri,
+			   gboolean allow_unknown_methods,
+			   gboolean allow_unsafe_methods,
+			   gboolean allow_transforms)
 {
 	GnomeVFSMethod *method;
 	GnomeVFSTransform *trans;
@@ -448,6 +456,12 @@ gnome_vfs_uri_new_private (const gchar *text_uri, gboolean allow_unknown_methods
 		return NULL;
 	}
 
+	method_scanner = get_method_string (text_uri, &method_string);
+	if (strcmp (method_string, "pipe") == 0 && !allow_unsafe_methods) {
+		g_free (method_string);
+		return NULL;
+	}
+
 	toplevel = g_new (GnomeVFSToplevelURI, 1);
 	toplevel->host_name = NULL;
 	toplevel->host_port = 0;
@@ -457,18 +471,19 @@ gnome_vfs_uri_new_private (const gchar *text_uri, gboolean allow_unknown_methods
 	uri = (GnomeVFSURI *) toplevel;
 	uri->parent = NULL;
 
-	method_scanner = get_method_string (text_uri, &method_string);
-	trans = gnome_vfs_transform_get (method_string);
-	if (trans != NULL && trans->transform) {
-		GnomeVFSContext *context;
-
-		context = gnome_vfs_context_new ();
-		(* trans->transform) (trans, method_scanner, &new_uri_string, context);
-		gnome_vfs_context_unref (context);
-		if (new_uri_string != NULL) {
-			toplevel->urn = g_strdup (text_uri);
-			g_free (method_string);
-			method_scanner = get_method_string (new_uri_string, &method_string);
+	if (allow_transforms) {
+		trans = gnome_vfs_transform_get (method_string);
+		if (trans != NULL && trans->transform) {
+			GnomeVFSContext *context;
+			
+			context = gnome_vfs_context_new ();
+			(* trans->transform) (trans, method_scanner, &new_uri_string, context);
+			gnome_vfs_context_unref (context);
+			if (new_uri_string != NULL) {
+				toplevel->urn = g_strdup (text_uri);
+				g_free (method_string);
+				method_scanner = get_method_string (new_uri_string, &method_string);
+			}
 		}
 	}
 	
@@ -1209,6 +1224,31 @@ gnome_vfs_uri_equal (const GnomeVFSURI *a,
 	    && string_match (toplevel_a->password, toplevel_b->password);
 }
 
+/* Convenience function that deals with the problem where we distinguish
+ * uris "foo://bar.com" and "foo://bar.com/" but we do not define
+ * what a child item of "foo://bar.com" would be -- to work around this,
+ * we will consider both "foo://bar.com" and "foo://bar.com/" the parent
+ * of "foo://bar.com/child"
+ */
+static gboolean
+uri_matches_as_parent (const GnomeVFSURI *possible_parent, const GnomeVFSURI *parent)
+{
+	GnomeVFSURI *alternate_possible_parent;
+	gboolean result;
+
+	if (strlen (possible_parent->text) == 0) {
+		alternate_possible_parent = gnome_vfs_uri_append_string (possible_parent,
+			GNOME_VFS_URI_PATH_STR);
+
+		result = gnome_vfs_uri_equal (alternate_possible_parent, parent);
+		
+		gnome_vfs_uri_unref (alternate_possible_parent);
+		return result;
+	}
+	
+	return gnome_vfs_uri_equal (possible_parent, parent);
+}
+
 /**
  * gnome_vfs_uri_is_parent:
  * @possible_parent: A GnomeVFSURI.
@@ -1237,7 +1277,7 @@ gnome_vfs_uri_is_parent (const GnomeVFSURI *possible_parent,
 			return FALSE;
 		}
 
-		result = gnome_vfs_uri_equal (item_parent_uri, possible_parent);	
+		result = uri_matches_as_parent (possible_parent, item_parent_uri);	
 		gnome_vfs_uri_unref (item_parent_uri);
 
 		return result;
@@ -1252,7 +1292,7 @@ gnome_vfs_uri_is_parent (const GnomeVFSURI *possible_parent,
 			return FALSE;
 		}
 
-		result = gnome_vfs_uri_equal (item_parent_uri, possible_parent);
+		result = uri_matches_as_parent (possible_parent, item_parent_uri);
 	
 		if (result) {
 			gnome_vfs_uri_unref (item_parent_uri);
