@@ -31,8 +31,17 @@
 #define _LARGEFILE64_SOURCE
 
 #include <glib.h>
+#if GNOME_PLATFORM_VERSION < 1095000
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#else
+/* FIXME: We need to use gettext, but we can't use the gettext helpers
+ * in libgnome since it depends on us, not the other way around.
+ * What's the good GNOME 2.0 solution for this?
+ */
+#define _(String) (String)
+#define N_(String) (String)
+#endif
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -592,12 +601,12 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 	       GnomeVFSFileInfoOptions options,
 	       struct stat *statptr)
 {
-	GnomeVFSResult result;
 	struct stat statbuf;
 	gboolean followed_symlink;
+	gboolean is_symlink;
 	gboolean recursive;
+	char *link_file_path;
 
-	result = GNOME_VFS_OK;
 	followed_symlink = FALSE;
 	
 	recursive = FALSE;
@@ -612,14 +621,14 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 		return gnome_vfs_result_from_errno ();
 	}
 
-	if ((options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) && S_ISLNK (statptr->st_mode)) {
+	is_symlink = S_ISLNK (statptr->st_mode);
+
+	if ((options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) && is_symlink) {
 		if (stat (full_name, statptr) != 0) {
 			if (errno == ELOOP) {
 				recursive = TRUE;
 			}
 
-			result = gnome_vfs_result_from_errno ();
-
 			/* It's a broken symlink, revert to the lstat. This is sub-optimal but
 			 * acceptable because it's not a common case.
 			 */
@@ -633,19 +642,41 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 
 	gnome_vfs_stat_to_file_info (file_info, statptr);
 
-	if (result == GNOME_VFS_OK && S_ISLNK (statptr->st_mode)) {
-		/* FIXME: call read_link in a loop unless recursive is TRUE.
-		 * Currently for more than one-level symlinks the symlink_name
-		 * field will be wrong when follow links is specified
+	if (is_symlink) {
+		link_file_path = g_strdup (full_name);
+		
+		/* We will either successfully read the link name or return
+		 * NULL if read_link fails -- flag it as a valid field either
+		 * way.
 		 */
-		file_info->symlink_name = read_link (full_name);
-		if (file_info->symlink_name == NULL) {
-			return gnome_vfs_result_from_errno ();
-		}
 		file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME;
+
+		while (TRUE) {			
+			/* Deal with multiple-level symlinks by following them as
+			 * far as we can.
+			 */
+
+			file_info->symlink_name = read_link (link_file_path);
+			if (file_info->symlink_name == NULL) {
+				g_free (link_file_path);
+				return gnome_vfs_result_from_errno ();
+			}
+			
+			if ((options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) == 0
+					/* we don't care to follow links */
+				|| lstat (file_info->symlink_name, statptr) != 0
+					/* we can't make out where this points to */
+				|| !S_ISLNK (statptr->st_mode)) {
+					/* the next level is not a link */
+				break;
+			}
+			g_free (link_file_path);
+			link_file_path = g_strdup (file_info->symlink_name);
+		}
+		g_free (link_file_path);
 	}
 
-	return result;
+	return GNOME_VFS_OK;
 }
 
 static GnomeVFSResult
