@@ -42,15 +42,22 @@
 #include <stdio.h> /* for sscanf */
 #include <string.h>
 
+#include <sys/types.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 
+#include "gnome-vfs-context.h"
+#include "gnome-vfs-types.h"
+#include "gnome-vfs-iobuf.h"
+#include "gnome-vfs-inet-connection.h"
+#include "gnome-vfs-method.h"
 #include "gnome-vfs-module.h"
 #include "gnome-vfs-module-shared.h"
 #include "gnome-vfs-mime.h"
+#include "gnome-vfs-parse-ls.h"
 
 #include "ftp-method.h"
 
@@ -71,12 +78,12 @@ static const char PROXY_KEY[] = "/system/gnome-vfs/http-proxy";
 static const char USE_PROXY_KEY[] = "/system/gnome-vfs/use-http-proxy";
 
 
-static GnomeVFSResult do_open	         (GnomeVFSMethod                *method,
+static GnomeVFSResult do_open	         (GnomeVFSMethod               *method,
 					  GnomeVFSMethodHandle         **method_handle,
 					  GnomeVFSURI                   *uri,
 					  GnomeVFSOpenMode               mode,
 					  GnomeVFSContext               *context);
-static gboolean       do_is_local        (GnomeVFSMethod                *method,
+static gboolean       do_is_local        (GnomeVFSMethod               *method,
 					  const GnomeVFSURI             *uri);
 static GnomeVFSResult do_open_directory  (GnomeVFSMethod                *method,
 					  GnomeVFSMethodHandle         **method_handle,
@@ -84,10 +91,10 @@ static GnomeVFSResult do_open_directory  (GnomeVFSMethod                *method,
 					  GnomeVFSFileInfoOptions        options,
 					  const GnomeVFSDirectoryFilter *filter,
 					  GnomeVFSContext               *context);
-static GnomeVFSResult do_close_directory (GnomeVFSMethod                *method,
+static GnomeVFSResult do_close_directory (GnomeVFSMethod               *method,
 					  GnomeVFSMethodHandle          *method_handle,
 					  GnomeVFSContext               *context);
-static GnomeVFSResult do_read_directory  (GnomeVFSMethod                *method,
+static GnomeVFSResult do_read_directory  (GnomeVFSMethod               *method,
 		                          GnomeVFSMethodHandle          *method_handle,
 		                          GnomeVFSFileInfo              *file_info,
 		                          GnomeVFSContext               *context);
@@ -96,7 +103,9 @@ static GnomeVFSResult do_read_directory  (GnomeVFSMethod                *method,
 
 guint                 ftp_connection_uri_hash  (gconstpointer c);
 gint                  ftp_connection_uri_equal (gconstpointer c, gconstpointer d);
-static GnomeVFSResult ftp_connection_acquire   (GnomeVFSURI *uri, FtpConnection **connection, GnomeVFSContext *context);
+static GnomeVFSResult ftp_connection_acquire   (GnomeVFSURI *uri, 
+		                                FtpConnection **connection, 
+		                                GnomeVFSContext *context);
 static void           ftp_connection_release   (FtpConnection *conn);
 
 
@@ -137,8 +146,10 @@ static void dircache_expire(GnomeVFSURI *uri) {
 }
 #endif
 
-#define ftp_debug(c,g) FTP_DEBUG((c),(g),__FILE__, __LINE__, __PRETTY_FUNCTION__)
 
+#if ENABLE_FTP_DEBUG
+
+#define ftp_debug(c,g) FTP_DEBUG((c),(g),__FILE__, __LINE__, __PRETTY_FUNCTION__)
 static void 
 FTP_DEBUG (FtpConnection *conn, 
 	   gchar *text, 
@@ -156,6 +167,12 @@ FTP_DEBUG (FtpConnection *conn,
 	g_free (text);
 }
 
+#else 
+
+#define ftp_debug(c,g) (g)
+
+#endif
+
 static GnomeVFSResult 
 ftp_response_to_vfs_result (FtpConnection *conn) 
 {
@@ -341,17 +358,11 @@ do_path_command (FtpConnection *conn,
 	char *actual_command;
 	GnomeVFSResult result;
 
-	/* FIXME: This doesn't interpret escape sequences, which is a
-	 * major problem. The FTP protocol doesn't use escape
-	 * sequences, so we should do the right kind of unescaping
-	 * (like file-method.c does).
-	 */
-
 	/* as some point we may need to make this execute a CD and then
   	 * a command using the basename rather than the full path. I am yet
 	 * to come across such a system.
 	 */
-	path = gnome_vfs_uri_get_path (uri);
+	path = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
 
 	if (path == NULL || strlen (path) == 0) {
 		path = ".";
@@ -420,7 +431,6 @@ do_transfer_command (FtpConnection *conn, gchar *command, GnomeVFSContext *conte
 	}
 
 	/* connect */
-	g_message ("Connect #2");
 	result = gnome_vfs_inet_connection_create (&conn->data_connection,
 						   host, 
 						   port,
@@ -469,7 +479,7 @@ do_path_transfer_command (FtpConnection *conn, gchar *command, GnomeVFSURI *uri,
   	 * a command using the basename rather than the full path. I am yet
 	 * to come across such a system.
 	 */
-	path = gnome_vfs_uri_get_path (uri);
+	path = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
 
 	if (path == NULL || strlen (path) == 0) {
 		path = "/";
@@ -918,8 +928,14 @@ ls_to_file_info (gchar *ls, GnomeVFSFileInfo *file_info,
 	if (filename) {
 
 		gnome_vfs_stat_to_file_info (file_info, &s);
-
-		/* we don't know the io_block size */
+		
+		/* FIXME: This is a hack, but we can't change
+		   the above API until after Gnome 1.4.  Ideally, we
+		   would give the stat_to_file_info function this
+		   information.  Also, there may be more fields here that are not 
+		   valid that we haven't dealt with.  */
+		file_info->valid_fields -= GNOME_VFS_FILE_INFO_FIELDS_DEVICE;
+		file_info->valid_fields -= GNOME_VFS_FILE_INFO_FIELDS_INODE;
 		file_info->valid_fields -= GNOME_VFS_FILE_INFO_FIELDS_IO_BLOCK_SIZE;
 		file_info->io_block_size = 0;
 
@@ -1123,7 +1139,7 @@ do_open_directory (GnomeVFSMethod *method,
 		return result;
 	}
 
-	g_print ("do_open_directory () in uri: %s\n", gnome_vfs_uri_get_path(uri));
+	/*g_print ("do_open_directory () in uri: %s\n", gnome_vfs_uri_get_path(uri));*/
 
 	if(strstr(conn->server_type,"MACOS")) {
 		/* don't ask for symlinks from MacOS servers */
@@ -1370,6 +1386,7 @@ do_set_file_info (GnomeVFSMethod *method,
 }
 
 static GnomeVFSMethod method = {
+	sizeof (GnomeVFSMethod),
 	do_open,
 	do_create,
 	do_close,
